<?php
/**
 * Created by WHMCSModule Networks (https://www.whmcsmodule.net/).
 *
 * PHP version 7.x and above
 *
 * @author WHMCSModule Networks <info@whmcsmodule.net>
 *
 * @author Zare (Hydra Communications Ltd) <team@zare.com>
 *
 * @see https://www.zare.com/
 *
 * @see Dcouemtnation https://zare.com/knowledgebase/article/75/whmcs-reseller-module-guide
 *
 */
namespace WHMCS\Module\Server\ZareServer;
if (!defined("WHMCS")) {die("This file cannot be accessed directly");}
use WHMCS\Database\Capsule;

/**
 * Class ZareServer
 *
 * Main class for Zare Servers in WHMCS.
 */
class ZareServer
{
  /**
 * @var string $secretKey The secret key used for authenticating API requests.
 */
  private $secretKey;

  /**
   * @var string $apiUrl The base URL of the API used for making requests.
   */
  private $apiUrl;

  /**
   * @var resource $curl The cURL handle used for making HTTP requests.
   */
  private $curl;

  /**
       * Constructor to initialize the API client.
       *
       * @param array $params Associative array containing configuration parameters.
       */
  public function __construct(array $params) {
      $this->secretKey = $params['serveraccesshash'];
      $this->apiUrl = 'https://manager.zare.com/api/v1/';
      $this->curl = curl_init();
  }

  /**
       * Executes a POST request to the API.
       *
       * @param string $apiUrl The API endpoint to request.
       * @param string $method The HTTP method to use.
       * @param array|null $postData The data to send in the request body.
       * @return array The response from the API.
       * @throws \Exception If there is an error with the cURL request.
       */
  public function callApi($apiUrl, $method, $postData = null) {
      curl_setopt($this->curl, CURLOPT_URL, $this->apiUrl.$apiUrl);
      curl_setopt($this->curl, CURLOPT_RETURNTRANSFER, true);
      if($method == "POST"){
        curl_setopt($this->curl, CURLOPT_POST, true);
      }
      if(isset($postData)){
        curl_setopt($this->curl, CURLOPT_POSTFIELDS, http_build_query($postData));
      }
      curl_setopt($this->curl, CURLOPT_HTTPHEADER, array(
          'Authorization: ' . $this->secretKey
      ));
      $response = curl_exec($this->curl);
      $statusCode = curl_getinfo($this->curl, CURLINFO_HTTP_CODE);
      if(curl_errno($this->curl)) {
          throw new \Exception('Curl error: ' . curl_error($this->curl));
      }
      curl_close($this->curl);
      $json_decode = json_decode($response, true);
        logModuleCall(
            'ZareServer',
            $method,
            $apiUrl.PHP_EOL.((!empty($postData)) ? json_encode($postData, JSON_PRETTY_PRINT) : ''),
            $json_decode,
            $json_decode,
            []
        );

        return ['response_code' => $statusCode,  'response' => $json_decode];
  }

  /**
       * Executes a GET request to the API.
       *
       * @param string $apiUrl The API endpoint to request.
       * @return array The response from the API.
       * @throws \Exception If there is an error with the cURL request.
       */
  public function callApiGet($apiUrl){
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_URL, $this->apiUrl.$apiUrl);
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($curl, CURLOPT_HTTPHEADER, array(
        'Authorization: ' . $this->secretKey
    ));
    $response = curl_exec($curl);
    if(curl_errno($curl)) {
        throw new \Exception('Curl error: ' . curl_error($curl));
    }
    curl_close($curl);
    $json_decode = json_decode($response, true);
    logModuleCall(
        'ZareServer',
        'GET',
        $apiUrl.PHP_EOL.(json_encode([], JSON_PRETTY_PRINT)),
        $json_decode,
        $json_decode,
        []
    );
    return $json_decode;
  }

  /**
       * Retrieves all dedicated servers.
       *
       * @return array The response from the API.
       */
  public function GetAllServers() {
      return $this->callApi('dedicated', 'POST');
  }

  /**
   * Retrieves data for a specific server.
   *
   * @param string $serverId The ID of the server.
   * @return array The response from the API.
   */
  public function GetServerData($serverId) {
      return $this->callApi('dedicated', 'POST', ['server_id' => $serverId]);
  }

  /**
   * Performs an action on a specific server.
   *
   * @param string $serverId The ID of the server.
   * @param string $serverAction The action to perform (e.g., powercycle, poweron, poweroff).
   * @return array The response from the API.
   */
  public function serverAction($serverId, $serverAction) {
      return $this->callApi('dedicated/'.$serverAction, 'POST', ['server_id' => $serverId]);
  }

  /**
   * Retrieves server bandwidth data.
   *
   * @param string $serverId The ID of the server.
   * @param string $type The type of bandwidth data (e.g., port_bits, port_percent).
   * @param string $scale The scale of the data (e.g., day, week, month).
   * @return array The response from the API.
   */
  public function serverBW($serverId, $type, $scale) {
    $ch = curl_init($this->apiUrl.'dedicated/graphs');
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_HTTPHEADER, [
        'Authorization: ' . $this->secretKey,
        'Content-Type: application/x-www-form-urlencoded'
    ]);
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query([
        'server_id' => $serverId,
        'type' => $type,
        'scale' => $scale
    ]));
    $response = curl_exec($ch);
    $statusCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    if(curl_errno($ch)) {
        throw new \Exception('Curl error: ' . curl_error($ch));
    } else {
      $responseData = json_decode($response, true);
    }
    curl_close($ch);

      logModuleCall(
          'ZareServer',
          'POST',
          'dedicated/graphs'.PHP_EOL.(json_encode([
              'server_id' => $serverId,
              'type' => $type,
              'scale' => $scale
          ], JSON_PRETTY_PRINT)),
          $responseData,
          $responseData,
          []
      );

      return ['response_code' => $statusCode,  'response' => $responseData];
  }

  /**
   * Retrieves server OS IDs.
   *
   * @return array The response from the API.
   */
  public function getServerOSIds() {
      return $this->callApi('dedicated/provision/os_ids', 'POST');
  }

  /**
   * Provisions a dedicated server with a specified OS.
   *
   * @param string $serverId The ID of the server.
   * @param string $osId The ID of the OS to provision.
   * @return array The response from the API.
   */
  public function provisionDedicatedServer($serverId, $osId) {
      return $this->callApi('dedicated/provision', 'POST', ['server_id' => $serverId, 'osid' => $osId]);
  }

  /**
   * Cancels a server.
   *
   * @param string $serverId The ID of the server.
   * @param string $type The type of cancellation (immediate, end, delete).
   * @return array The response from the API.
   */
  public function CancelServer($serverId, $type) {
      return $this->callApi('reseller/billing/cancellation', 'POST', ['server_id' => $serverId, 'type' => $type]);
  }

  /**
   * Retrieves firewall history for a server.
   *
   * @param string $serverId The ID of the server.
   * @return array The response from the API.
   */
  public function getFirewallHistory($serverId) {
      return $this->callApi('dedicated/firewall', 'POST', ['server_id' => $serverId, 'scale' => "week"]);
  }

  /**
   * Sets the sensitivity of the firewall for a specific IP address.
   *
   * @param string $serverId The ID of the server.
   * @param string $ip The IP address.
   * @param string $sensitivity The sensitivity level high, medium, low, normal.
   * @return array The response from the API.
   */
  public function SetFirewallSensitivity($serverId, $ip, $sensitivity) {
      return $this->callApi('dedicated/firewall/sensitivity', 'POST', ['server_id' => $serverId, 'ip' => $ip, 'sensitivity' => $sensitivity]);
  }

  /**
   * Sets the state of the firewall for a specific IP address.
   *
   * @param string $serverId The ID of the server.
   * @param string $ip The IP address.
   * @param string $state The state (e.g., Always On (1), Redirect On Attack (0)).
   * @return array The response from the API.
   */
  public function SetFirewallState($serverId, $ip, $state) {
      return $this->callApi('dedicated/firewall/state', 'POST', ['server_id' => $serverId, 'ip' => $ip, 'state' => $state]);
  }

  /**
   * Modifies the PTR record for an IP address.
   *
   * @param string $serverId The ID of the server.
   * @param string $ip The IP address.
   * @param string $ptr The PTR record to set.
   * @return array The response from the API.
   */
  public function ModifyIPaddress($serverId, $ip, $ptr) {
      return $this->callApi('dedicated/dns', 'POST', ['server_id' => $serverId, 'ip' => $ip, 'type' => "ptr", "content" => $ptr]);
  }

  /**
   * Retrieves locations.
   *
   * @return array The list of locations.
   */
  public function getLocations() {
    return $this->callApiGet('reseller/locations')['locations'];
  }

  /**
   * Retrieves console access URL for a server.
   *
   * @param string $serverId The ID of the server.
   * @return string The console access URL.
   * @throws \Exception If there is an error with the cURL request.
   */
  public function getConsole($serverId) {
    $url = 'https://manager.zare.com/api/v1/dedicated/console';
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query(array('server_id' => $serverId)));
    curl_setopt($ch, CURLOPT_HTTPHEADER, array(
        'Authorization: ' . $this->secretKey
    ));
    $response = curl_exec($ch);
    if(curl_errno($ch)) {
      throw new \Exception('Curl error: ' . curl_error($ch));
    }
    curl_close($ch);
    return $response;
  }

  /**
   * Converts terabytes to gigabytes.
   *
   * @param float $terabytes The size in terabytes.
   * @return float The size in gigabytes.
   */
public function formatSizeTBtoGB($terabytes)
{
    return round($terabytes * 1024, 2);
}

/**
 * Converts terabytes to megabytes.
 *
 * @param float $terabytes The size in terabytes.
 * @return float The size in megabytes.
 */
public function formatSizeTBtoMB($terabytes)
{
    return round($terabytes * 1024 * 1024, 2);
}

/**
 * Converts gigabytes to megabytes.
 *
 * @param float $gigabytes The size in gigabytes.
 * @return float The size in megabytes.
 */
public function formatSizeGBtoMB($gigabytes)
{
    return round($gigabytes * 1024, 2);
}

/**
 * Converts an amount from one currency to another.
 *
 * @param string $fromCurrency The currency code to convert from.
 * @param string $toCurrency The currency code to convert to.
 * @param float $amount The amount to convert.
 * @return array The converted amount.
 */
public function CurrencyConvert($fromCurrency, $toCurrency, $amount)
{
$amount_from_db = Capsule::table('tblcurrencies')->where('code', $toCurrency)->get();
$exhangeRate_whmcs = json_decode($amount_from_db, true)[0]['rate'];
$convertedAmount = $amount * $exhangeRate_whmcs;
return array(  'convertedAmount' => round($convertedAmount, 2) );
}

/**
 * Matches an OS name to a predefined platform.
 *
 * @param string $OS The OS name to match.
 * @return string The matched platform.
 */
public function OSMatch($OS) {
    $platforms = [
        '/CentOS/' => 'centos',
        '/Arch Linux/' => 'archlinux',
        '/Fedora/' => 'fedora',
        '/openSUSE|Suse/' => 'suse',
        '/Nextcloud/' => 'nextcloud',
        '/LAMP/' => 'lamp',
        '/Debian/' => 'debian',
        '/Ubuntu/' => 'ubuntu',
        '/cPanel/' => 'cpanel',
        '/Plesk/' => 'plesk',
        '/Windows/' => 'windows',
        '/Alma/' => 'alma',
        '/Rocky/' => 'rocky',
        '/FreeBSD/' => 'freebsd',
        '/Scientific/' => 'scientific',
        '/Docker/' => 'docker',
        '/Webuzo/' => 'webuzo',
        '/Openvpn/' => 'openvpn',
    ];

    foreach ($platforms as $pattern => $platform) {
        if (preg_match($pattern, $OS)) {
            return $platform;
        }
    }

    return 'misc';
}

}
