<?php
/**
 * Created by WHMCSModule Networks (https://www.whmcsmodule.net/).
 *
 * PHP version 7.x and above
 *
 * @author WHMCSModule Networks <info@whmcsmodule.net>
 *
 * @author Zare (Hydra Communications Ltd) <team@zare.com>
 *
 * @see https://www.zare.com/
 *
 * @see Dcouemtnation https://zare.com/knowledgebase/article/75/whmcs-reseller-module-guide
 *
 */
namespace WHMCS\Module\Server\ZareServer;
if (!defined("WHMCS")) {die("This file cannot be accessed directly");}
use WHMCS\Database\Capsule;
use WHMCS\Module\Server\ZareServer\ZareServer;
use WHMCS\Module\Server\ZareServer\Helper;
use WHMCS\Product\Product;
use App;

/**
 * Class configOptions
 *
 * Manages the configuration options for Zare Servers in WHMCS.
 * Handles the creation and management of configurable options, custom fields,
 * and location settings for products.
 */
class configOptions
{
  /** @var ZareServer $manager API client for interacting with ZareServer. */
  private $manager;

  /** @var array $params Parameters for initializing the class. */
  private $params;

  /** @var array $_LANG Language array for translation. */
  private $_LANG;

  /**
   * Constructor for configOptions.
   *
   * Initializes the ZareServer client and sets up class parameters and language array.
   *
   * @param array $params Parameters including configuration details for ZareServer.
   */
  public function __construct(array $params)
  {
    $this->manager = new ZareServer($params);
    $this->params = $params;
    $this->_LANG = Helper::getLang();
  }

  /**
   * Main method to handle configuration options.
   *
   * Based on the request action, ensures custom fields are set and creates configurable options
   * for the product. Returns an empty array.
   *
   * @return array An empty array.
   */
  public function configs(){
    $product = Product::find(App::getFromRequest('id'));
    if (App::getFromRequest('action') == 'save') {
    $this->ensureCustomFields($product);
    $this->createGlobalConfigurableOptions($product);
    }
    return [];
  }

  /**
   * Creates global configurable options for the given product.
   *
   * Checks if a configuration group exists. If not, creates it and associates it with the product.
   * Then, creates template fields and location fields for the group.
   *
   * @param Product $product The product for which to create configurable options.
   * @return void
   */
  private function createGlobalConfigurableOptions($product)
  {
    $groupId = Capsule::table('tblproductconfiggroups')->where('name', 'Configurable options for Zare Servers')->first()->id;
    $currencyId = Capsule::table('tblcurrencies')->where('default', '1')->first()->id;
    $currencyCode = Capsule::table('tblcurrencies')->where('default', '1')->first()->code;
    if(!$groupId){
      $groupId = Capsule::table('tblproductconfiggroups')->insertGetId(['name' => 'Configurable options for Zare Servers', 'description' => 'Auto generated by Zare Servers module']);
      $groupIdLinks = Capsule::table('tblproductconfiglinks')->where('gid', $groupId)->where('pid', $product->id)->first()->gid;
      if(!$groupIdLinks){
        Capsule::table('tblproductconfiglinks')->insert(['gid' => $groupId, 'pid' => $product->id]);
      }
        $this->createTemplateFields($groupId, $currencyId, $currencyCode);
        $this->createLocationFields($groupId);
    }
  }

  /**
   * Creates template fields for the specified configuration group.
   *
   * Retrieves templates from the ZareServer and inserts them into the product configuration options.
   * Updates pricing based on the template type (Windows or other).
   *
   * @param int $groupId The ID of the configuration group.
   * @param int $currencyId The ID of the currency.
   * @param string $currencyCode The currency code.
   * @return void
   */
  private function createTemplateFields($groupId, $currencyId, $currencyCode){
    $optionId = Capsule::table('tblproductconfigoptions')->where('gid', $groupId)->where('optionname', 'template|Template')->first()->id;
    if(!$optionId){
      $optionId = Capsule::table('tblproductconfigoptions')->insertGetId(['gid' => $groupId, 'optionname' => 'template|Template', 'optiontype' => 1]);
      $templates = $this->manager->getServerOSIds()['response'];

      foreach ($templates as $template) {
          Capsule::table('tblproductconfigoptionssub')->updateOrInsert(['optionname' => $template['id'].'|'.$template['name']], ['configid' => $optionId]);
      }
      foreach (Capsule::table('tblproductconfigoptionssub')->where('configid', $optionId)->get() as $id) {
        if (strpos($id->optionname, "Windows") !== false) {
          $monthlys = $this->manager->CurrencyConvert('GBP', $currencyCode, "28");
        } else {
          $monthlys['convertedAmount'] = "0.00";
        }
          Capsule::table('tblpricing')->updateOrInsert(['type' => 'configoptions', 'relid' => $id->id, 'monthly' => $monthlys['convertedAmount']],  ['currency' => $currencyId]);
      }
    }
  }

  /**
   * Ensures that custom fields are created for the product if they do not exist.
   *
   * Creates custom fields for server ID and rescue password.
   *
   * @param Product $product The product for which to ensure custom fields.
   * @return void
   */
  private function ensureCustomFields($product)
{
    $customFields = [
        ['fieldname' => 'serverId|Server Id', 'fieldtype' => 'text', 'adminonly' => 'on'],
        ['fieldname' => 'rescue|Rescue Password', 'fieldtype' => 'text', 'adminonly' => 'on'],
    ];

    foreach ($customFields as $field) {
        if (empty(Capsule::table('tblcustomfields')->where('fieldname', $field['fieldname'])->where('relid', $product->id)->value('relid'))) {
            Capsule::table('tblcustomfields')->updateOrInsert(['type' => 'product', 'relid' => $product->id] + $field);
        }
    }
}

/**
  * Creates location fields for the specified configuration group.
  *
  * Retrieves location zones from ZareServer and inserts them into the product configuration options.
  *
  * @param int $groupId The ID of the configuration group.
  * @return void
  */
  private function createLocationFields($groupId){
    $zones = $this->manager->getLocations();
      $optionId = Capsule::table('tblproductconfigoptions')->where('gid', $groupId)->where('optionname', 'location|Location')->first()->id;
        if(!$optionId){
          $optionId = Capsule::table('tblproductconfigoptions')->insertGetId(['gid' => $groupId, 'optionname' => 'location|Location', 'optiontype' => 1]);
          foreach ($zones as $zone) {
              Capsule::table('tblproductconfigoptionssub')->updateOrInsert(['optionname' => $zone['id'].'|'.$zone['long_name']], ['configid' => $optionId]);
          }
        }
  }


}
